import { API as API$1 } from '../../../scw/api.js';
import { urlParams, validatePathParam } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { marshalCreateProjectRequest, unmarshalProject, unmarshalListProjectsResponse, marshalUpdateProjectRequest } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Account API.
 *
 * User related data. This API allows you to manage projects.
 */
class API extends API$1 {
  /**
   * Create a new Project for an Organization. Deprecated in favor of Account
   * API v3. Generate a new Project for an Organization, specifying its
   * configuration including name and description.
   *
   * @deprecated
   * @param request - The request {@link CreateProjectRequest}
   * @returns A Promise of Project
   */
  createProject = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        body: JSON.stringify(marshalCreateProjectRequest(request, _this.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/account/v2/projects`
      }, unmarshalProject);
    };
  })();
  pageOfListProjects = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this2.client.fetch({
        method: 'GET',
        path: `/account/v2/projects`,
        urlParams: urlParams(['name', request.name], ['order_by', request.orderBy ?? 'created_at_asc'], ['organization_id', request.organizationId ?? _this2.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? _this2.client.settings.defaultPageSize], ['project_ids', request.projectIds])
      }, unmarshalListProjectsResponse);
    };
  })();

  /**
   * List all Projects of an Organization. Deprecated in favor of Account API
   * v3. List all Projects of an Organization. The response will include the
   * total number of Projects as well as their associated Organizations, names
   * and IDs. Other information include the creation and update date of the
   * Project.
   *
   * @deprecated
   * @param request - The request {@link ListProjectsRequest}
   * @returns A Promise of ListProjectsResponse
   */
  listProjects = (() => {
    var _this3 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('projects', _this3.pageOfListProjects, request);
    };
  })();

  /**
   * Get an existing Project. Deprecated in favor of Account API v3. Retrieve
   * information about an existing Project, specified by its Project ID. Its
   * full details, including ID, name and description, are returned in the
   * response object.
   *
   * @deprecated
   * @param request - The request {@link GetProjectRequest}
   * @returns A Promise of Project
   */
  getProject = (() => {
    var _this4 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this4.client.fetch({
        method: 'GET',
        path: `/account/v2/projects/${validatePathParam('projectId', request.projectId ?? _this4.client.settings.defaultProjectId)}`
      }, unmarshalProject);
    };
  })();

  /**
   * Delete an existing Project. Deprecated in favor of Account API v3. Delete
   * an existing Project, specified by its Project ID. The Project needs to be
   * empty (meaning there are no resources left in it) to be deleted
   * effectively. Note that deleting a Project is permanent, and cannot be
   * undone.
   *
   * @deprecated
   * @param request - The request {@link DeleteProjectRequest}
   */
  deleteProject = (() => {
    var _this5 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this5.client.fetch({
        method: 'DELETE',
        path: `/account/v2/projects/${validatePathParam('projectId', request.projectId ?? _this5.client.settings.defaultProjectId)}`
      });
    };
  })();

  /**
   * Update Project. Deprecated in favor of Account API v3. Update the
   * parameters of an existing Project, specified by its Project ID. These
   * parameters include the name and description.
   *
   * @deprecated
   * @param request - The request {@link UpdateProjectRequest}
   * @returns A Promise of Project
   */
  updateProject = (() => {
    var _this6 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this6.client.fetch({
        body: JSON.stringify(marshalUpdateProjectRequest(request, _this6.client.settings)),
        headers: jsonContentHeaders,
        method: 'PATCH',
        path: `/account/v2/projects/${validatePathParam('projectId', request.projectId ?? _this6.client.settings.defaultProjectId)}`
      }, unmarshalProject);
    };
  })();
}

export { API };
