import { API as API$1 } from '../../../scw/api.js';
import { validatePathParam, urlParams } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { unmarshalGetServerTypesAvailabilityResponse, unmarshalListServersTypesResponse, unmarshalListVolumesTypesResponse, unmarshalListServersResponse, marshalCreateServerRequest, unmarshalCreateServerResponse, unmarshalGetServerResponse, marshalSetServerRequest, unmarshalSetServerResponse, marshalUpdateServerRequest, unmarshalUpdateServerResponse, unmarshalListServerActionsResponse, marshalServerActionRequest, unmarshalServerActionResponse, unmarshalListServerUserDataResponse, unmarshalListImagesResponse, unmarshalGetImageResponse, marshalCreateImageRequest, unmarshalCreateImageResponse, marshalSetImageRequest, unmarshalSetImageResponse, unmarshalListSnapshotsResponse, marshalCreateSnapshotRequest, unmarshalCreateSnapshotResponse, unmarshalGetSnapshotResponse, marshalSetSnapshotRequest, unmarshalSetSnapshotResponse, marshalExportSnapshotRequest, unmarshalExportSnapshotResponse, unmarshalListVolumesResponse, marshalCreateVolumeRequest, unmarshalCreateVolumeResponse, unmarshalGetVolumeResponse, marshalUpdateVolumeRequest, unmarshalUpdateVolumeResponse, unmarshalListSecurityGroupsResponse, marshalCreateSecurityGroupRequest, unmarshalCreateSecurityGroupResponse, unmarshalGetSecurityGroupResponse, marshalSetSecurityGroupRequest, unmarshalSetSecurityGroupResponse, unmarshalListSecurityGroupRulesResponse, marshalCreateSecurityGroupRuleRequest, unmarshalCreateSecurityGroupRuleResponse, marshalSetSecurityGroupRulesRequest, unmarshalSetSecurityGroupRulesResponse, unmarshalGetSecurityGroupRuleResponse, marshalSetSecurityGroupRuleRequest, unmarshalSetSecurityGroupRuleResponse, unmarshalListPlacementGroupsResponse, marshalCreatePlacementGroupRequest, unmarshalCreatePlacementGroupResponse, unmarshalGetPlacementGroupResponse, marshalSetPlacementGroupRequest, unmarshalSetPlacementGroupResponse, marshalUpdatePlacementGroupRequest, unmarshalUpdatePlacementGroupResponse, unmarshalGetPlacementGroupServersResponse, marshalSetPlacementGroupServersRequest, unmarshalSetPlacementGroupServersResponse, marshalUpdatePlacementGroupServersRequest, unmarshalUpdatePlacementGroupServersResponse, unmarshalListIpsResponse, marshalCreateIpRequest, unmarshalCreateIpResponse, unmarshalGetIpResponse, marshalUpdateIpRequest, unmarshalUpdateIpResponse, unmarshalListPrivateNICsResponse, marshalCreatePrivateNICRequest, unmarshalCreatePrivateNICResponse, unmarshalGetPrivateNICResponse, marshalUpdatePrivateNICRequest, unmarshalPrivateNIC, unmarshalListBootscriptsResponse, unmarshalGetBootscriptResponse, unmarshalGetDashboardResponse, marshalPlanBlockMigrationRequest, unmarshalMigrationPlan, marshalApplyBlockMigrationRequest } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Instance API. */
class API extends API$1 {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'fr-par-3', 'nl-ams-1', 'nl-ams-2', 'nl-ams-3', 'pl-waw-1', 'pl-waw-2', 'pl-waw-3'];

  /**
   * Get availability. Get availability for all Instance types.
   *
   * @param request - The request {@link GetServerTypesAvailabilityRequest}
   * @returns A Promise of GetServerTypesAvailabilityResponse
   */
  getServerTypesAvailability = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this.client.settings.defaultZone)}/products/servers/availability`,
        urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? _this.client.settings.defaultPageSize])
      }, unmarshalGetServerTypesAvailabilityResponse);
    };
  })();

  /**
   * List Instance types. List available Instance types and their technical
   * details.
   *
   * @param request - The request {@link ListServersTypesRequest}
   * @returns A Promise of ListServersTypesResponse
   */
  listServersTypes = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this2.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this2.client.settings.defaultZone)}/products/servers`,
        urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? _this2.client.settings.defaultPageSize])
      }, unmarshalListServersTypesResponse);
    };
  })();

  /**
   * List volume types. List all volume types and their technical details.
   *
   * @param request - The request {@link ListVolumesTypesRequest}
   * @returns A Promise of ListVolumesTypesResponse
   */
  listVolumesTypes = (() => {
    var _this3 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this3.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this3.client.settings.defaultZone)}/products/volumes`,
        urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? _this3.client.settings.defaultPageSize])
      }, unmarshalListVolumesTypesResponse);
    };
  })();
  pageOfListServers = (() => {
    var _this4 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this4.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this4.client.settings.defaultZone)}/servers`,
        urlParams: urlParams(['commercial_type', request.commercialType], ['name', request.name], ['order', request.order], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this4.client.settings.defaultPageSize], ['private_ip', request.privateIp], ['private_network', request.privateNetwork], ['private_networks', request.privateNetworks && request.privateNetworks.length > 0 ? request.privateNetworks.join(',') : undefined], ['private_nic_mac_address', request.privateNicMacAddress], ['project', request.project], ['servers', request.servers && request.servers.length > 0 ? request.servers.join(',') : undefined], ['state', request.state], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined], ['without_ip', request.withoutIp])
      }, unmarshalListServersResponse);
    };
  })();

  /**
   * List all Instances. List all Instances in a specified Availability Zone,
   * e.g. `fr-par-1`.
   *
   * @param request - The request {@link ListServersRequest}
   * @returns A Promise of ListServersResponse
   */
  listServers = (() => {
    var _this5 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('servers', _this5.pageOfListServers, request);
    };
  })();
  _createServer = request => this.client.fetch({
    body: JSON.stringify(marshalCreateServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`
  }, unmarshalCreateServerResponse);

  /**
   * Delete an Instance. Delete the Instance with the specified ID.
   *
   * @param request - The request {@link DeleteServerRequest}
   */
  deleteServer = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  });

  /**
   * Get an Instance. Get the details of a specified Instance.
   *
   * @param request - The request {@link GetServerRequest}
   * @returns A Promise of GetServerResponse
   */
  getServer = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalGetServerResponse);
  _setServer = request => this.client.fetch({
    body: JSON.stringify(marshalSetServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('id', request.id)}`
  }, unmarshalSetServerResponse);
  _updateServer = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalUpdateServerResponse);

  /**
   * List Instance actions. List all actions (e.g. power on, power off, reboot)
   * that can currently be performed on an Instance.
   *
   * @param request - The request {@link ListServerActionsRequest}
   * @returns A Promise of ListServerActionsResponse
   */
  listServerActions = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/action`
  }, unmarshalListServerActionsResponse);

  /**
   * Perform action. Perform an action on an Instance. Available actions are:
   * `poweron`: Start a stopped Instance. `poweroff`: Fully stop the Instance
   * and release the hypervisor slot. `stop_in_place`: Stop the Instance, but
   * keep the slot on the hypervisor. `reboot`: Stop the instance and restart
   * it. `backup`: Create an image with all the volumes of an Instance.
   * `terminate`: Delete the Instance along with all attached volumes.
   * `enable_routed_ip`: Migrate the Instance to the new network stack.
   *
   * Keep in mind that terminating an Instance will result in the deletion of
   * all attached volumes, including local and block storage. If you want to
   * preserve your local volumes, you should use the `archive` action instead of
   * `terminate`. Similarly, if you want to keep your block storage volumes, you
   * must first detach them before issuing the `terminate` command. For more
   * information, read the [Volumes](#path-volumes-list-volumes) documentation.
   *
   * @param request - The request {@link ServerActionRequest}
   * @returns A Promise of ServerActionResponse
   */
  serverAction = request => this.client.fetch({
    body: JSON.stringify(marshalServerActionRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/action`
  }, unmarshalServerActionResponse);

  /**
   * List user data. List all user data keys registered on a specified Instance.
   *
   * @param request - The request {@link ListServerUserDataRequest}
   * @returns A Promise of ListServerUserDataResponse
   */
  listServerUserData = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/user_data`
  }, unmarshalListServerUserDataResponse);

  /**
   * Delete user data. Delete the specified key from an Instance's user data.
   *
   * @param request - The request {@link DeleteServerUserDataRequest}
   */
  deleteServerUserData = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/user_data/${validatePathParam('key', request.key)}`
  });
  pageOfListImages = (() => {
    var _this6 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this6.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this6.client.settings.defaultZone)}/images`,
        urlParams: urlParams(['arch', request.arch], ['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this6.client.settings.defaultPageSize], ['project', request.project], ['public', request.public], ['tags', request.tags])
      }, unmarshalListImagesResponse);
    };
  })();

  /**
   * List Instance images. List all existing Instance images.
   *
   * @param request - The request {@link ListImagesRequest}
   * @returns A Promise of ListImagesResponse
   */
  listImages = (() => {
    var _this7 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('images', _this7.pageOfListImages, request);
    };
  })();

  /**
   * Get an Instance image. Get details of an image with the specified ID.
   *
   * @param request - The request {@link GetImageRequest}
   * @returns A Promise of GetImageResponse
   */
  getImage = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalGetImageResponse);

  /**
   * Create an Instance image. Create an Instance image from the specified
   * snapshot ID.
   *
   * @param request - The request {@link CreateImageRequest}
   * @returns A Promise of CreateImageResponse
   */
  createImage = request => this.client.fetch({
    body: JSON.stringify(marshalCreateImageRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images`
  }, unmarshalCreateImageResponse);
  _setImage = request => this.client.fetch({
    body: JSON.stringify(marshalSetImageRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('id', request.id)}`
  }, unmarshalSetImageResponse);

  /**
   * Delete an Instance image. Delete the image with the specified ID.
   *
   * @param request - The request {@link DeleteImageRequest}
   */
  deleteImage = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('imageId', request.imageId)}`
  });
  pageOfListSnapshots = (() => {
    var _this8 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this8.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this8.client.settings.defaultZone)}/snapshots`,
        urlParams: urlParams(['base_volume_id', request.baseVolumeId], ['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this8.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags])
      }, unmarshalListSnapshotsResponse);
    };
  })();

  /**
   * List snapshots. List all snapshots of an Organization in a specified
   * Availability Zone.
   *
   * @param request - The request {@link ListSnapshotsRequest}
   * @returns A Promise of ListSnapshotsResponse
   */
  listSnapshots = (() => {
    var _this9 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('snapshots', _this9.pageOfListSnapshots, request);
    };
  })();

  /**
   * Create a snapshot from a specified volume or from a QCOW2 file. Create a
   * snapshot from a specified volume or from a QCOW2 file in a specified
   * Availability Zone.
   *
   * @param request - The request {@link CreateSnapshotRequest}
   * @returns A Promise of CreateSnapshotResponse
   */
  createSnapshot = (() => {
    var _this10 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this10.client.fetch({
        body: JSON.stringify(marshalCreateSnapshotRequest(request, _this10.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this10.client.settings.defaultZone)}/snapshots`
      }, unmarshalCreateSnapshotResponse);
    };
  })();

  /**
   * Get a snapshot. Get details of a snapshot with the specified ID.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @returns A Promise of GetSnapshotResponse
   */
  getSnapshot = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalGetSnapshotResponse);
  _setSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalSetSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSetSnapshotResponse);

  /**
   * Delete a snapshot. Delete the snapshot with the specified ID.
   *
   * @param request - The request {@link DeleteSnapshotRequest}
   */
  deleteSnapshot = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  });

  /**
   * Export a snapshot. Export a snapshot to a specified S3 bucket in the same
   * region.
   *
   * @param request - The request {@link ExportSnapshotRequest}
   * @returns A Promise of ExportSnapshotResponse
   */
  exportSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalExportSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}/export`
  }, unmarshalExportSnapshotResponse);
  pageOfListVolumes = (() => {
    var _this11 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this11.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this11.client.settings.defaultZone)}/volumes`,
        urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this11.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined], ['volume_type', request.volumeType])
      }, unmarshalListVolumesResponse);
    };
  })();

  /**
   * List volumes. List volumes in the specified Availability Zone. You can
   * filter the output by volume type.
   *
   * @param request - The request {@link ListVolumesRequest}
   * @returns A Promise of ListVolumesResponse
   */
  listVolumes = (() => {
    var _this12 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('volumes', _this12.pageOfListVolumes, request);
    };
  })();

  /**
   * Create a volume. Create a volume of a specified type in an Availability
   * Zone.
   *
   * @param request - The request {@link CreateVolumeRequest}
   * @returns A Promise of CreateVolumeResponse
   */
  createVolume = (() => {
    var _this13 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this13.client.fetch({
        body: JSON.stringify(marshalCreateVolumeRequest(request, _this13.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this13.client.settings.defaultZone)}/volumes`
      }, unmarshalCreateVolumeResponse);
    };
  })();

  /**
   * Get a volume. Get details of a volume with the specified ID.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @returns A Promise of GetVolumeResponse
   */
  getVolume = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalGetVolumeResponse);

  /**
   * Update a volume. Replace the name and/or size properties of a volume
   * specified by its ID, with the specified value(s). Any volume name can be
   * changed, however only `b_ssd` volumes can currently be increased in size.
   *
   * @param request - The request {@link UpdateVolumeRequest}
   * @returns A Promise of UpdateVolumeResponse
   */
  updateVolume = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalUpdateVolumeResponse);

  /**
   * Delete a volume. Delete the volume with the specified ID.
   *
   * @param request - The request {@link DeleteVolumeRequest}
   */
  deleteVolume = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  });
  pageOfListSecurityGroups = (() => {
    var _this14 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this14.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this14.client.settings.defaultZone)}/security_groups`,
        urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this14.client.settings.defaultPageSize], ['project', request.project], ['project_default', request.projectDefault], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined])
      }, unmarshalListSecurityGroupsResponse);
    };
  })();

  /**
   * List security groups. List all existing security groups.
   *
   * @param request - The request {@link ListSecurityGroupsRequest}
   * @returns A Promise of ListSecurityGroupsResponse
   */
  listSecurityGroups = (() => {
    var _this15 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('securityGroups', _this15.pageOfListSecurityGroups, request);
    };
  })();

  /**
   * Create a security group. Create a security group with a specified name and
   * description.
   *
   * @param request - The request {@link CreateSecurityGroupRequest}
   * @returns A Promise of CreateSecurityGroupResponse
   */
  createSecurityGroup = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecurityGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups`
  }, unmarshalCreateSecurityGroupResponse);

  /**
   * Get a security group. Get the details of a security group with the
   * specified ID.
   *
   * @param request - The request {@link GetSecurityGroupRequest}
   * @returns A Promise of GetSecurityGroupResponse
   */
  getSecurityGroup = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}`
  }, unmarshalGetSecurityGroupResponse);

  /**
   * Delete a security group. Delete a security group with the specified ID.
   *
   * @param request - The request {@link DeleteSecurityGroupRequest}
   */
  deleteSecurityGroup = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}`
  });
  _setSecurityGroup = request => this.client.fetch({
    body: JSON.stringify(marshalSetSecurityGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('id', request.id)}`
  }, unmarshalSetSecurityGroupResponse);

  /**
   * Get default rules. Lists the default rules applied to all the security
   * groups.
   *
   * @param request - The request {@link ListDefaultSecurityGroupRulesRequest}
   * @returns A Promise of ListSecurityGroupRulesResponse
   */
  listDefaultSecurityGroupRules = (() => {
    var _this16 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this16.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this16.client.settings.defaultZone)}/security_groups/default/rules`
      }, unmarshalListSecurityGroupRulesResponse);
    };
  })();
  pageOfListSecurityGroupRules = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize])
  }, unmarshalListSecurityGroupRulesResponse);

  /**
   * List rules. List the rules of the a specified security group ID.
   *
   * @param request - The request {@link ListSecurityGroupRulesRequest}
   * @returns A Promise of ListSecurityGroupRulesResponse
   */
  listSecurityGroupRules = request => enrichForPagination('rules', this.pageOfListSecurityGroupRules, request);

  /**
   * Create rule. Create a rule in the specified security group ID.
   *
   * @param request - The request {@link CreateSecurityGroupRuleRequest}
   * @returns A Promise of CreateSecurityGroupRuleResponse
   */
  createSecurityGroupRule = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecurityGroupRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules`
  }, unmarshalCreateSecurityGroupRuleResponse);

  /**
   * Update all the rules of a security group. Replaces the existing rules of
   * the security group with the rules provided. This endpoint supports the
   * update of existing rules, creation of new rules and deletion of existing
   * rules when they are not passed in the request.
   *
   * @param request - The request {@link SetSecurityGroupRulesRequest}
   * @returns A Promise of SetSecurityGroupRulesResponse
   */
  setSecurityGroupRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetSecurityGroupRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules`
  }, unmarshalSetSecurityGroupRulesResponse);

  /**
   * Delete rule. Delete a security group rule with the specified ID.
   *
   * @param request - The request {@link DeleteSecurityGroupRuleRequest}
   */
  deleteSecurityGroupRule = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  });

  /**
   * Get rule. Get details of a security group rule with the specified ID.
   *
   * @param request - The request {@link GetSecurityGroupRuleRequest}
   * @returns A Promise of GetSecurityGroupRuleResponse
   */
  getSecurityGroupRule = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  }, unmarshalGetSecurityGroupRuleResponse);
  _setSecurityGroupRule = request => this.client.fetch({
    body: JSON.stringify(marshalSetSecurityGroupRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  }, unmarshalSetSecurityGroupRuleResponse);
  pageOfListPlacementGroups = (() => {
    var _this17 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this17.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this17.client.settings.defaultZone)}/placement_groups`,
        urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this17.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined])
      }, unmarshalListPlacementGroupsResponse);
    };
  })();

  /**
   * List placement groups. List all placement groups in a specified
   * Availability Zone.
   *
   * @param request - The request {@link ListPlacementGroupsRequest}
   * @returns A Promise of ListPlacementGroupsResponse
   */
  listPlacementGroups = (() => {
    var _this18 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('placementGroups', _this18.pageOfListPlacementGroups, request);
    };
  })();

  /**
   * Create a placement group. Create a new placement group in a specified
   * Availability Zone.
   *
   * @param request - The request {@link CreatePlacementGroupRequest}
   * @returns A Promise of CreatePlacementGroupResponse
   */
  createPlacementGroup = (() => {
    var _this19 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this19.client.fetch({
        body: JSON.stringify(marshalCreatePlacementGroupRequest(request, _this19.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this19.client.settings.defaultZone)}/placement_groups`
      }, unmarshalCreatePlacementGroupResponse);
    };
  })();

  /**
   * Get a placement group. Get the specified placement group.
   *
   * @param request - The request {@link GetPlacementGroupRequest}
   * @returns A Promise of GetPlacementGroupResponse
   */
  getPlacementGroup = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  }, unmarshalGetPlacementGroupResponse);

  /**
   * Set placement group. Set all parameters of the specified placement group.
   *
   * @param request - The request {@link SetPlacementGroupRequest}
   * @returns A Promise of SetPlacementGroupResponse
   */
  setPlacementGroup = request => this.client.fetch({
    body: JSON.stringify(marshalSetPlacementGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  }, unmarshalSetPlacementGroupResponse);

  /**
   * Update a placement group. Update one or more parameter of the specified
   * placement group.
   *
   * @param request - The request {@link UpdatePlacementGroupRequest}
   * @returns A Promise of UpdatePlacementGroupResponse
   */
  updatePlacementGroup = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePlacementGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  }, unmarshalUpdatePlacementGroupResponse);

  /**
   * Delete the specified placement group.
   *
   * @param request - The request {@link DeletePlacementGroupRequest}
   */
  deletePlacementGroup = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  });

  /**
   * Get placement group servers. Get all Instances belonging to the specified
   * placement group.
   *
   * @param request - The request {@link GetPlacementGroupServersRequest}
   * @returns A Promise of GetPlacementGroupServersResponse
   */
  getPlacementGroupServers = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}/servers`
  }, unmarshalGetPlacementGroupServersResponse);

  /**
   * Set placement group servers. Set all Instances belonging to the specified
   * placement group.
   *
   * @param request - The request {@link SetPlacementGroupServersRequest}
   * @returns A Promise of SetPlacementGroupServersResponse
   */
  setPlacementGroupServers = request => this.client.fetch({
    body: JSON.stringify(marshalSetPlacementGroupServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}/servers`
  }, unmarshalSetPlacementGroupServersResponse);

  /**
   * Update placement group servers. Update all Instances belonging to the
   * specified placement group.
   *
   * @param request - The request {@link UpdatePlacementGroupServersRequest}
   * @returns A Promise of UpdatePlacementGroupServersResponse
   */
  updatePlacementGroupServers = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePlacementGroupServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}/servers`
  }, unmarshalUpdatePlacementGroupServersResponse);
  pageOfListIps = (() => {
    var _this20 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this20.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this20.client.settings.defaultZone)}/ips`,
        urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? _this20.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined], ['type', request.type])
      }, unmarshalListIpsResponse);
    };
  })();

  /**
   * List all flexible IPs. List all flexible IPs in a specified zone.
   *
   * @param request - The request {@link ListIpsRequest}
   * @returns A Promise of ListIpsResponse
   */
  listIps = (() => {
    var _this21 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('ips', _this21.pageOfListIps, request);
    };
  })();

  /**
   * Reserve a flexible IP. Reserve a flexible IP and attach it to the specified
   * Instance.
   *
   * @param request - The request {@link CreateIpRequest}
   * @returns A Promise of CreateIpResponse
   */
  createIp = (() => {
    var _this22 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this22.client.fetch({
        body: JSON.stringify(marshalCreateIpRequest(request, _this22.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this22.client.settings.defaultZone)}/ips`
      }, unmarshalCreateIpResponse);
    };
  })();

  /**
   * Get a flexible IP. Get details of an IP with the specified ID or address.
   *
   * @param request - The request {@link GetIpRequest}
   * @returns A Promise of GetIpResponse
   */
  getIp = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ip', request.ip)}`
  }, unmarshalGetIpResponse);

  /**
   * Update a flexible IP. Update a flexible IP in the specified zone with the
   * specified ID.
   *
   * @param request - The request {@link UpdateIpRequest}
   * @returns A Promise of UpdateIpResponse
   */
  updateIp = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ip', request.ip)}`
  }, unmarshalUpdateIpResponse);

  /**
   * Delete a flexible IP. Delete the IP with the specified ID.
   *
   * @param request - The request {@link DeleteIpRequest}
   */
  deleteIp = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ip', request.ip)}`
  });
  pageOfListPrivateNICs = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined])
  }, unmarshalListPrivateNICsResponse);

  /**
   * List all private NICs. List all private NICs of a specified Instance.
   *
   * @param request - The request {@link ListPrivateNICsRequest}
   * @returns A Promise of ListPrivateNICsResponse
   */
  listPrivateNICs = request => enrichForPagination('privateNics', this.pageOfListPrivateNICs, request);

  /**
   * Create a private NIC connecting an Instance to a Private Network.
   *
   * @param request - The request {@link CreatePrivateNICRequest}
   * @returns A Promise of CreatePrivateNICResponse
   */
  createPrivateNIC = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePrivateNICRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics`
  }, unmarshalCreatePrivateNICResponse);

  /**
   * Get a private NIC. Get private NIC properties.
   *
   * @param request - The request {@link GetPrivateNICRequest}
   * @returns A Promise of GetPrivateNICResponse
   */
  getPrivateNIC = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics/${validatePathParam('privateNicId', request.privateNicId)}`
  }, unmarshalGetPrivateNICResponse);

  /**
   * Update a private NIC. Update one or more parameter(s) of a specified
   * private NIC.
   *
   * @param request - The request {@link UpdatePrivateNICRequest}
   * @returns A Promise of PrivateNIC
   */
  updatePrivateNIC = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePrivateNICRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics/${validatePathParam('privateNicId', request.privateNicId)}`
  }, unmarshalPrivateNIC);

  /**
   * Delete a private NIC.
   *
   * @param request - The request {@link DeletePrivateNICRequest}
   */
  deletePrivateNIC = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics/${validatePathParam('privateNicId', request.privateNicId)}`
  });
  pageOfListBootscripts = (() => {
    var _this23 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this23.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this23.client.settings.defaultZone)}/bootscripts`,
        urlParams: urlParams(['arch', request.arch], ['default', request.default], ['page', request.page], ['per_page', request.perPage ?? _this23.client.settings.defaultPageSize], ['public', request.public], ['title', request.title])
      }, unmarshalListBootscriptsResponse);
    };
  })();

  /**
   * List bootscripts.
   *
   * @deprecated
   * @param request - The request {@link ListBootscriptsRequest}
   * @returns A Promise of ListBootscriptsResponse
   */
  listBootscripts = (() => {
    var _this24 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('bootscripts', _this24.pageOfListBootscripts, request);
    };
  })();

  /**
   * Get bootscripts. Get details of a bootscript with the specified ID.
   *
   * @deprecated
   * @param request - The request {@link GetBootscriptRequest}
   * @returns A Promise of GetBootscriptResponse
   */
  getBootscript = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/bootscripts/${validatePathParam('bootscriptId', request.bootscriptId)}`
  }, unmarshalGetBootscriptResponse);
  getDashboard = (() => {
    var _this25 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this25.client.fetch({
        method: 'GET',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this25.client.settings.defaultZone)}/dashboard`,
        urlParams: urlParams(['organization', request.organization], ['project', request.project])
      }, unmarshalGetDashboardResponse);
    };
  })();

  /**
   * Get a volume or snapshot's migration plan. Given a volume or snapshot,
   * returns the migration plan for a call to the RPC ApplyBlockMigration. This
   * plan will include zero or one volume, and zero or more snapshots, which
   * will need to be migrated together. This RPC does not perform the actual
   * migration itself, ApplyBlockMigration must be used. The validation_key
   * value returned by this call must be provided to the ApplyBlockMigration
   * call to confirm that all resources listed in the plan should be migrated.
   *
   * @param request - The request {@link PlanBlockMigrationRequest}
   * @returns A Promise of MigrationPlan
   */
  planBlockMigration = (() => {
    var _this26 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this26.client.fetch({
        body: JSON.stringify(marshalPlanBlockMigrationRequest(request, _this26.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? _this26.client.settings.defaultZone)}/block-migration/plan`
      }, unmarshalMigrationPlan);
    };
  })();

  /**
   * Migrate a volume and/or snapshots to SBS (Scaleway Block Storage). To be
   * used, this RPC must be preceded by a call to PlanBlockMigration. To migrate
   * all resources mentioned in the MigrationPlan, the validation_key returned
   * in the MigrationPlan must be provided.
   *
   * @param request - The request {@link ApplyBlockMigrationRequest}
   */
  applyBlockMigration = request => this.client.fetch({
    body: JSON.stringify(marshalApplyBlockMigrationRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/block-migration/apply`
  });
}

export { API };
