import { API as API$1 } from '../../../scw/api.js';
import { validatePathParam, urlParams } from '../../../helpers/marshalling.js';
import { enrichForPagination } from '../../../scw/fetch/resource-paginator.js';
import { marshalNatsApiCreateNatsAccountRequest, unmarshalNatsAccount, marshalNatsApiUpdateNatsAccountRequest, unmarshalListNatsAccountsResponse, marshalNatsApiCreateNatsCredentialsRequest, unmarshalNatsCredentials, unmarshalListNatsCredentialsResponse, marshalSnsApiActivateSnsRequest, unmarshalSnsInfo, marshalSnsApiDeactivateSnsRequest, marshalSnsApiCreateSnsCredentialsRequest, unmarshalSnsCredentials, marshalSnsApiUpdateSnsCredentialsRequest, unmarshalListSnsCredentialsResponse, marshalSqsApiActivateSqsRequest, unmarshalSqsInfo, marshalSqsApiDeactivateSqsRequest, marshalSqsApiCreateSqsCredentialsRequest, unmarshalSqsCredentials, marshalSqsApiUpdateSqsCredentialsRequest, unmarshalListSqsCredentialsResponse } from './marshalling.gen.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Messaging and Queuing NATS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing NATS accounts.
 * Messaging and Queuing NATS API.
 */
class NatsAPI extends API$1 {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];

  /**
   * Create a NATS account. Create a NATS account associated with a Project.
   *
   * @param request - The request {@link NatsApiCreateNatsAccountRequest}
   * @returns A Promise of NatsAccount
   */
  createNatsAccount = (() => {
    var _this = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this.client.fetch({
        body: JSON.stringify(marshalNatsApiCreateNatsAccountRequest(request, _this.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this.client.settings.defaultRegion)}/nats-accounts`
      }, unmarshalNatsAccount);
    };
  })();

  /**
   * Delete a NATS account. Delete a NATS account, specified by its NATS account
   * ID. Note that deleting a NATS account is irreversible, and any credentials,
   * streams, consumer and stored messages belonging to this NATS account will
   * also be deleted.
   *
   * @param request - The request {@link NatsApiDeleteNatsAccountRequest}
   */
  deleteNatsAccount = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts/${validatePathParam('natsAccountId', request.natsAccountId)}`
  });

  /**
   * Update the name of a NATS account. Update the name of a NATS account,
   * specified by its NATS account ID.
   *
   * @param request - The request {@link NatsApiUpdateNatsAccountRequest}
   * @returns A Promise of NatsAccount
   */
  updateNatsAccount = request => this.client.fetch({
    body: JSON.stringify(marshalNatsApiUpdateNatsAccountRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts/${validatePathParam('natsAccountId', request.natsAccountId)}`
  }, unmarshalNatsAccount);

  /**
   * Get a NATS account. Retrieve information about an existing NATS account
   * identified by its NATS account ID. Its full details, including name and
   * endpoint, are returned in the response.
   *
   * @param request - The request {@link NatsApiGetNatsAccountRequest}
   * @returns A Promise of NatsAccount
   */
  getNatsAccount = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts/${validatePathParam('natsAccountId', request.natsAccountId)}`
  }, unmarshalNatsAccount);
  pageOfListNatsAccounts = (() => {
    var _this2 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this2.client.fetch({
        method: 'GET',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this2.client.settings.defaultRegion)}/nats-accounts`,
        urlParams: urlParams(['order_by', request.orderBy ?? 'created_at_asc'], ['page', request.page], ['page_size', request.pageSize ?? _this2.client.settings.defaultPageSize], ['project_id', request.projectId])
      }, unmarshalListNatsAccountsResponse);
    };
  })();

  /**
   * List NATS accounts. List all NATS accounts in the specified region, for a
   * Scaleway Organization or Project. By default, the NATS accounts returned in
   * the list are ordered by creation date in ascending order, though this can
   * be modified via the `order_by` field.
   *
   * @param request - The request {@link NatsApiListNatsAccountsRequest}
   * @returns A Promise of ListNatsAccountsResponse
   */
  listNatsAccounts = (() => {
    var _this3 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('natsAccounts', _this3.pageOfListNatsAccounts, request);
    };
  })();

  /**
   * Create NATS credentials. Create a set of credentials for a NATS account,
   * specified by its NATS account ID.
   *
   * @param request - The request {@link NatsApiCreateNatsCredentialsRequest}
   * @returns A Promise of NatsCredentials
   */
  createNatsCredentials = request => this.client.fetch({
    body: JSON.stringify(marshalNatsApiCreateNatsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials`
  }, unmarshalNatsCredentials);

  /**
   * Delete NATS credentials. Delete a set of credentials, specified by their
   * credentials ID. Deleting credentials is irreversible and cannot be undone.
   * The credentials can no longer be used to access the NATS account, and
   * active connections using this credentials will be closed.
   *
   * @param request - The request {@link NatsApiDeleteNatsCredentialsRequest}
   */
  deleteNatsCredentials = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials/${validatePathParam('natsCredentialsId', request.natsCredentialsId)}`
  });

  /**
   * Get NATS credentials. Retrieve an existing set of credentials, identified
   * by the `nats_credentials_id`. The credentials themselves are NOT returned,
   * only their metadata (NATS account ID, credentials name, etc), are returned
   * in the response.
   *
   * @param request - The request {@link NatsApiGetNatsCredentialsRequest}
   * @returns A Promise of NatsCredentials
   */
  getNatsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials/${validatePathParam('natsCredentialsId', request.natsCredentialsId)}`
  }, unmarshalNatsCredentials);
  pageOfListNatsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials`,
    urlParams: urlParams(['nats_account_id', request.natsAccountId], ['order_by', request.orderBy ?? 'created_at_asc'], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListNatsCredentialsResponse);

  /**
   * List NATS credentials. List existing credentials in the specified NATS
   * account. The response contains only the metadata for the credentials, not
   * the credentials themselves, which are only returned after a **Create
   * Credentials** call.
   *
   * @param request - The request {@link NatsApiListNatsCredentialsRequest}
   * @returns A Promise of ListNatsCredentialsResponse
   */
  listNatsCredentials = request => enrichForPagination('natsCredentials', this.pageOfListNatsCredentials, request);
}

/**
 * Messaging and Queuing SNS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing SNS brokers.
 * Messaging and Queuing SNS API.
 */
class SnsAPI extends API$1 {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];

  /**
   * Activate SNS. Activate SNS for the specified Project ID. SNS must be
   * activated before any usage. Activating SNS does not trigger any billing,
   * and you can deactivate at any time.
   *
   * @param request - The request {@link SnsApiActivateSnsRequest}
   * @returns A Promise of SnsInfo
   */
  activateSns = (() => {
    var _this4 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this4.client.fetch({
        body: JSON.stringify(marshalSnsApiActivateSnsRequest(request, _this4.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this4.client.settings.defaultRegion)}/activate-sns`
      }, unmarshalSnsInfo);
    };
  })();

  /**
   * Get SNS info. Retrieve the SNS information of the specified Project ID.
   * Informations include the activation status and the SNS API endpoint URL.
   *
   * @param request - The request {@link SnsApiGetSnsInfoRequest}
   * @returns A Promise of SnsInfo
   */
  getSnsInfo = (() => {
    var _this5 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this5.client.fetch({
        method: 'GET',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this5.client.settings.defaultRegion)}/sns-info`,
        urlParams: urlParams(['project_id', request.projectId ?? _this5.client.settings.defaultProjectId])
      }, unmarshalSnsInfo);
    };
  })();

  /**
   * Deactivate SNS. Deactivate SNS for the specified Project ID.You must delete
   * all topics and credentials before this call or you need to set the
   * force_delete parameter.
   *
   * @param request - The request {@link SnsApiDeactivateSnsRequest}
   * @returns A Promise of SnsInfo
   */
  deactivateSns = (() => {
    var _this6 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this6.client.fetch({
        body: JSON.stringify(marshalSnsApiDeactivateSnsRequest(request, _this6.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this6.client.settings.defaultRegion)}/deactivate-sns`
      }, unmarshalSnsInfo);
    };
  })();

  /**
   * Create SNS credentials. Create a set of credentials for SNS, specified by a
   * Project ID. Credentials give the bearer access to topics, and the level of
   * permissions can be defined granularly.
   *
   * @param request - The request {@link SnsApiCreateSnsCredentialsRequest}
   * @returns A Promise of SnsCredentials
   */
  createSnsCredentials = (() => {
    var _this7 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this7.client.fetch({
        body: JSON.stringify(marshalSnsApiCreateSnsCredentialsRequest(request, _this7.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this7.client.settings.defaultRegion)}/sns-credentials`
      }, unmarshalSnsCredentials);
    };
  })();

  /**
   * Delete SNS credentials. Delete a set of SNS credentials, specified by their
   * credentials ID. Deleting credentials is irreversible and cannot be undone.
   * The credentials can then no longer be used to access SNS.
   *
   * @param request - The request {@link SnsApiDeleteSnsCredentialsRequest}
   */
  deleteSnsCredentials = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials/${validatePathParam('snsCredentialsId', request.snsCredentialsId)}`
  });

  /**
   * Update SNS credentials. Update a set of SNS credentials. You can update the
   * credentials' name, or their permissions.
   *
   * @param request - The request {@link SnsApiUpdateSnsCredentialsRequest}
   * @returns A Promise of SnsCredentials
   */
  updateSnsCredentials = request => this.client.fetch({
    body: JSON.stringify(marshalSnsApiUpdateSnsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials/${validatePathParam('snsCredentialsId', request.snsCredentialsId)}`
  }, unmarshalSnsCredentials);

  /**
   * Get SNS credentials. Retrieve an existing set of credentials, identified by
   * the `credentials_id`. The credentials themselves, as well as their metadata
   * (name, project ID etc), are returned in the response.
   *
   * @param request - The request {@link SnsApiGetSnsCredentialsRequest}
   * @returns A Promise of SnsCredentials
   */
  getSnsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials/${validatePathParam('snsCredentialsId', request.snsCredentialsId)}`
  }, unmarshalSnsCredentials);
  pageOfListSnsCredentials = (() => {
    var _this8 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this8.client.fetch({
        method: 'GET',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this8.client.settings.defaultRegion)}/sns-credentials`,
        urlParams: urlParams(['order_by', request.orderBy ?? 'created_at_asc'], ['page', request.page], ['page_size', request.pageSize ?? _this8.client.settings.defaultPageSize], ['project_id', request.projectId])
      }, unmarshalListSnsCredentialsResponse);
    };
  })();

  /**
   * List SNS credentials. List existing SNS credentials in the specified
   * region. The response contains only the metadata for the credentials, not
   * the credentials themselves.
   *
   * @param request - The request {@link SnsApiListSnsCredentialsRequest}
   * @returns A Promise of ListSnsCredentialsResponse
   */
  listSnsCredentials = (() => {
    var _this9 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('snsCredentials', _this9.pageOfListSnsCredentials, request);
    };
  })();
}

/**
 * Messaging and Queuing SQS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing SQS brokers.
 * Messaging and Queuing SQS API.
 */
class SqsAPI extends API$1 {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];

  /**
   * Activate SQS. Activate SQS for the specified Project ID. SQS must be
   * activated before any usage such as creating credentials and queues.
   * Activating SQS does not trigger any billing, and you can deactivate at any
   * time.
   *
   * @param request - The request {@link SqsApiActivateSqsRequest}
   * @returns A Promise of SqsInfo
   */
  activateSqs = (() => {
    var _this10 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this10.client.fetch({
        body: JSON.stringify(marshalSqsApiActivateSqsRequest(request, _this10.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this10.client.settings.defaultRegion)}/activate-sqs`
      }, unmarshalSqsInfo);
    };
  })();

  /**
   * Get SQS info. Retrieve the SQS information of the specified Project ID.
   * Informations include the activation status and the SQS API endpoint URL.
   *
   * @param request - The request {@link SqsApiGetSqsInfoRequest}
   * @returns A Promise of SqsInfo
   */
  getSqsInfo = (() => {
    var _this11 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this11.client.fetch({
        method: 'GET',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this11.client.settings.defaultRegion)}/sqs-info`,
        urlParams: urlParams(['project_id', request.projectId ?? _this11.client.settings.defaultProjectId])
      }, unmarshalSqsInfo);
    };
  })();

  /**
   * Deactivate SQS. Deactivate SQS for the specified Project ID. You must
   * delete all queues and credentials before this call or you need to set the
   * force_delete parameter.
   *
   * @param request - The request {@link SqsApiDeactivateSqsRequest}
   * @returns A Promise of SqsInfo
   */
  deactivateSqs = (() => {
    var _this12 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this12.client.fetch({
        body: JSON.stringify(marshalSqsApiDeactivateSqsRequest(request, _this12.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this12.client.settings.defaultRegion)}/deactivate-sqs`
      }, unmarshalSqsInfo);
    };
  })();

  /**
   * Create SQS credentials. Create a set of credentials for SQS, specified by a
   * Project ID. Credentials give the bearer access to queues, and the level of
   * permissions can be defined granularly.
   *
   * @param request - The request {@link SqsApiCreateSqsCredentialsRequest}
   * @returns A Promise of SqsCredentials
   */
  createSqsCredentials = (() => {
    var _this13 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this13.client.fetch({
        body: JSON.stringify(marshalSqsApiCreateSqsCredentialsRequest(request, _this13.client.settings)),
        headers: jsonContentHeaders,
        method: 'POST',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this13.client.settings.defaultRegion)}/sqs-credentials`
      }, unmarshalSqsCredentials);
    };
  })();

  /**
   * Delete SQS credentials. Delete a set of SQS credentials, specified by their
   * credentials ID. Deleting credentials is irreversible and cannot be undone.
   * The credentials can then no longer be used to access SQS.
   *
   * @param request - The request {@link SqsApiDeleteSqsCredentialsRequest}
   */
  deleteSqsCredentials = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials/${validatePathParam('sqsCredentialsId', request.sqsCredentialsId)}`
  });

  /**
   * Update SQS credentials. Update a set of SQS credentials. You can update the
   * credentials' name, or their permissions.
   *
   * @param request - The request {@link SqsApiUpdateSqsCredentialsRequest}
   * @returns A Promise of SqsCredentials
   */
  updateSqsCredentials = request => this.client.fetch({
    body: JSON.stringify(marshalSqsApiUpdateSqsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials/${validatePathParam('sqsCredentialsId', request.sqsCredentialsId)}`
  }, unmarshalSqsCredentials);

  /**
   * Get SQS credentials. Retrieve an existing set of credentials, identified by
   * the `credentials_id`. The credentials themselves, as well as their metadata
   * (name, project ID etc), are returned in the response.
   *
   * @param request - The request {@link SqsApiGetSqsCredentialsRequest}
   * @returns A Promise of SqsCredentials
   */
  getSqsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials/${validatePathParam('sqsCredentialsId', request.sqsCredentialsId)}`
  }, unmarshalSqsCredentials);
  pageOfListSqsCredentials = (() => {
    var _this14 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return _this14.client.fetch({
        method: 'GET',
        path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? _this14.client.settings.defaultRegion)}/sqs-credentials`,
        urlParams: urlParams(['order_by', request.orderBy ?? 'created_at_asc'], ['page', request.page], ['page_size', request.pageSize ?? _this14.client.settings.defaultPageSize], ['project_id', request.projectId])
      }, unmarshalListSqsCredentialsResponse);
    };
  })();

  /**
   * List SQS credentials. List existing SQS credentials in the specified
   * region. The response contains only the metadata for the credentials, not
   * the credentials themselves.
   *
   * @param request - The request {@link SqsApiListSqsCredentialsRequest}
   * @returns A Promise of ListSqsCredentialsResponse
   */
  listSqsCredentials = (() => {
    var _this15 = this;
    return function (request) {
      if (request === void 0) {
        request = {};
      }
      return enrichForPagination('sqsCredentials', _this15.pageOfListSqsCredentials, request);
    };
  })();
}

export { NatsAPI, SnsAPI, SqsAPI };
