import { isJSONObject } from '../../../helpers/json.js';
import { unmarshalDate, unmarshalArrayOfObject, resolveOneOf } from '../../../helpers/marshalling.js';

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalFolder = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Folder' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    path: data.path,
    projectId: data.project_id
  };
};
const unmarshalSecret = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Secret' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    id: data.id,
    isManaged: data.is_managed,
    isProtected: data.is_protected,
    name: data.name,
    path: data.path,
    projectId: data.project_id,
    region: data.region,
    status: data.status,
    tags: data.tags,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    versionCount: data.version_count
  };
};
const unmarshalSecretVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecretVersion' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    isLatest: data.is_latest,
    revision: data.revision,
    secretId: data.secret_id,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalAccessSecretVersionResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AccessSecretVersionResponse' failed as data isn't a dictionary.`);
  }
  return {
    data: data.data,
    dataCrc32: data.data_crc32,
    revision: data.revision,
    secretId: data.secret_id
  };
};
const unmarshalListFoldersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFoldersResponse' failed as data isn't a dictionary.`);
  }
  return {
    folders: unmarshalArrayOfObject(data.folders, unmarshalFolder),
    totalCount: data.total_count
  };
};
const unmarshalListSecretVersionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecretVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    versions: unmarshalArrayOfObject(data.versions, unmarshalSecretVersion)
  };
};
const unmarshalListSecretsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecretsResponse' failed as data isn't a dictionary.`);
  }
  return {
    secrets: unmarshalArrayOfObject(data.secrets, unmarshalSecret),
    totalCount: data.total_count
  };
};
const unmarshalListTagsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTagsResponse' failed as data isn't a dictionary.`);
  }
  return {
    tags: data.tags,
    totalCount: data.total_count
  };
};
const marshalPasswordGenerationParams = (request, defaults) => ({
  additional_chars: request.additionalChars,
  length: request.length,
  no_digits: request.noDigits,
  no_lowercase_letters: request.noLowercaseLetters,
  no_uppercase_letters: request.noUppercaseLetters
});
const marshalAddSecretOwnerRequest = (request, defaults) => ({
  product: request.product ?? 'unknown',
  product_name: request.productName
});
const marshalCreateFolderRequest = (request, defaults) => ({
  name: request.name,
  path: request.path,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalCreateSecretRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  path: request.path,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags,
  type: request.type ?? 'unknown_secret_type'
});
const marshalCreateSecretVersionRequest = (request, defaults) => ({
  data: request.data,
  data_crc32: request.dataCrc32,
  description: request.description,
  disable_previous: request.disablePrevious,
  ...resolveOneOf([{
    param: 'password_generation',
    value: request.passwordGeneration ? marshalPasswordGenerationParams(request.passwordGeneration) : undefined
  }])
});
const marshalGeneratePasswordRequest = (request, defaults) => ({
  additional_chars: request.additionalChars,
  description: request.description,
  disable_previous: request.disablePrevious,
  length: request.length,
  no_digits: request.noDigits,
  no_lowercase_letters: request.noLowercaseLetters,
  no_uppercase_letters: request.noUppercaseLetters
});
const marshalUpdateSecretRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  path: request.path,
  tags: request.tags
});
const marshalUpdateSecretVersionRequest = (request, defaults) => ({
  description: request.description
});

export { marshalAddSecretOwnerRequest, marshalCreateFolderRequest, marshalCreateSecretRequest, marshalCreateSecretVersionRequest, marshalGeneratePasswordRequest, marshalUpdateSecretRequest, marshalUpdateSecretVersionRequest, unmarshalAccessSecretVersionResponse, unmarshalFolder, unmarshalListFoldersResponse, unmarshalListSecretVersionsResponse, unmarshalListSecretsResponse, unmarshalListTagsResponse, unmarshalSecret, unmarshalSecretVersion };
