import { ConsoleLogger } from './console-logger.js';

let sdkLogger = /*#__PURE__*/new ConsoleLogger('silent');

/**
 * Sets a logger to be used within the SDK.
 *
 * @param logger - The Logger instance
 *
 * @public
 */
const setLogger = logger => {
  sdkLogger = logger;
};

/**
 * Sets the logger to console logger with given logLevel (log is disabled by default).
 *
 * @param logLevel - The Log level (default to 'warn')
 * @param prefix - A Log message prefix (default to 'scaleway-sdk-js:')
 *
 * @public
 */
const enableConsoleLogger = function (logLevel, prefix) {
  if (logLevel === void 0) {
    logLevel = 'warn';
  }
  if (prefix === void 0) {
    prefix = 'scaleway-sdk-js:';
  }
  return setLogger(new ConsoleLogger(logLevel, prefix));
};

/**
 * Returns the active SDK logger.
 *
 * @internal
 */
const getLogger = () => sdkLogger;

export { enableConsoleLogger, getLogger, setLogger };
