import { isAccessKey, isSecretKey } from '../internal/validations/string-validation.js';

/**
 * Holds access key and secret key.
 *
 * @public
 */

/**
 * Holds default values of a Scaleway profile.
 *
 * @public
 */

/**
 * Holds values of a Scaleway profile.
 *
 * @public
 */

/**
 * Verifies that the payload contains both the accessKey and the secretKey.
 *
 * @param obj - The secrets
 * @returns Whether the secrets are not empty.
 *
 * @internal
 */
const hasAuthenticationSecrets = obj => typeof obj.accessKey === 'string' && obj.accessKey !== '' && typeof obj.secretKey === 'string' && obj.secretKey !== '';

/**
 * Asserts the format of secrets.
 *
 * @param obj - The secrets
 * @returns Whether the secrets use a valid format
 *
 * @throws Error
 * Thrown if either the accessKey or the secretKey has en invalid format.
 *
 * @internal
 */
function assertValidAuthenticationSecrets(obj) {
  if (!(obj.accessKey && obj.secretKey)) {
    throw new Error(`Invalid secrets, accessKey & secretKey must be defined. See https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/`);
  }
  if (!isAccessKey(obj.accessKey)) {
    throw new Error(`Invalid access key format '${obj.accessKey}', expected SCWXXXXXXXXXXXXXXXXX format. See https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/`);
  }
  if (!isSecretKey(obj.secretKey)) {
    throw new Error(`Invalid secret key format '${obj.secretKey}', expected a UUID: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx. See https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/`);
  }
}

export { assertValidAuthenticationSecrets, hasAuthenticationSecrets };
